commit aebd0dfdfbafee16957c3c615d3706bb6aab952d
Author: Alex Osborne <bobofdoom@gmail.com>
Date:   Thu Aug 2 21:31:26 2007 +1000

    Add a flag to allow LFCR as a valid linebreak.
---
 include/gsmd/gsmd.h   |    2 ++
 src/gsmd/atcmd.c      |   22 ++++++++++++++++------
 src/gsmd/vendor_bcm.c |    3 +++
 3 files changed, 21 insertions(+), 6 deletions(-)

Index: gsm/include/gsmd/gsmd.h
===================================================================
--- gsm.orig/include/gsmd/gsmd.h	2007-08-15 00:35:50.000000000 +0800
+++ gsm/include/gsmd/gsmd.h	2007-08-15 00:36:46.000000000 +0800
@@ -17,6 +17,7 @@
 /* Refer to 3GPP TS 07.07 v 7.8.0, Chapter 4.1 */
 #define LGSM_ATCMD_F_EXTENDED	0x01	/* as opposed to basic */
 #define LGSM_ATCMD_F_PARAM	0x02	/* as opposed to action */
+#define LGSM_ATCMD_F_LFCR	0x04	/* accept LFCR as a line terminator */
 
 struct gsmd_atcmd {
 	struct llist_head list;
@@ -37,6 +38,7 @@
 	LLPARSE_STATE_IDLE_LF,		/* LF before response (V1) */
 	LLPARSE_STATE_RESULT,		/* within result payload */
 	LLPARSE_STATE_RESULT_CR,	/* CR after result */
+	LLPARSE_STATE_RESULT_LF,	/* LF after result */
 	LLPARSE_STATE_PROMPT,		/* within a "> " prompt */
 	LLPARSE_STATE_PROMPT_SPC,	/* a complete "> " prompt */
 	LLPARSE_STATE_ERROR,		/* something went wrong */
Index: gsm/src/gsmd/atcmd.c
===================================================================
--- gsm.orig/src/gsmd/atcmd.c	2007-08-15 00:35:50.000000000 +0800
+++ gsm/src/gsmd/atcmd.c	2007-08-15 00:36:46.000000000 +0800
@@ -76,6 +76,14 @@
 	}
 }
 
+static inline void llparse_endline(struct llparser *llp)
+{
+	/* re-set cursor to start of buffer */
+	llp->cur = llp->buf;
+	llp->state = LLPARSE_STATE_IDLE;
+	memset(llp->buf, 0, LLPARSE_BUF_SIZE);
+}
+
 static int llparse_byte(struct llparser *llp, char byte)
 {
 	int ret = 0;
@@ -121,16 +129,18 @@
 	case LLPARSE_STATE_RESULT:
 		if (byte == '\r')
 			llp->state = LLPARSE_STATE_RESULT_CR;
+		else if ((llp->flags & LGSM_ATCMD_F_LFCR) && byte == '\n')
+			llp->state = LLPARSE_STATE_RESULT_LF;
 		else
 			ret = llparse_append(llp, byte);
 		break;
 	case LLPARSE_STATE_RESULT_CR:
-		if (byte == '\n') {
-			/* re-set cursor to start of buffer */
-			llp->cur = llp->buf;
-			llp->state = LLPARSE_STATE_IDLE;
-			memset(llp->buf, 0, LLPARSE_BUF_SIZE);
-		}
+		if (byte == '\n')
+			llparse_endline(llp);
+		break;
+	case LLPARSE_STATE_RESULT_LF:
+		if (byte == '\r')
+			llparse_endline(llp);
 		break;
 	case LLPARSE_STATE_PROMPT:
 		if (byte == ' ')
Index: gsm/src/gsmd/vendor_bcm.c
===================================================================
--- gsm.orig/src/gsmd/vendor_bcm.c	2007-08-15 00:36:27.000000000 +0800
+++ gsm/src/gsmd/vendor_bcm.c	2007-08-15 00:36:46.000000000 +0800
@@ -98,6 +98,9 @@
 	int rc;
 	struct gsmd_atcmd *cmd;
 
+	/* bcm sometimes sends LFCR instead of CRLF (eg *MTSMENU message) */
+	g->llp.flags |= LGSM_ATCMD_F_LFCR;
+
 	/* TODO */
 	return rc;
 }
